from app import db, socketio
from app.events import connected_sessions
from app.models import Corpus
from flask import current_app, request
from flask_login import login_required
from .CQiClient.CQiClient import CQiClient
import logging


analysis_clients = {}


@socketio.on('init_corpus_analysis')
@login_required
def init_corpus_analysis(corpus_id):
    ''' TODO: Check if current_user is allowed to subscribe to this '''
    socketio.start_background_task(observe_corpus_analysis_connection,
                                   current_app._get_current_object(),
                                   corpus_id,
                                   request.sid)


@socketio.on('query_event')
def recv_query(message):
    logger = logging.getLogger(__name__)
    logger.warning(message)
    analysis_client = analysis_clients[request.sid]


def observe_corpus_analysis_connection(app, corpus_id, session_id):
    logger = logging.getLogger(__name__)
    with app.app_context():
        corpus = Corpus.query.filter_by(id=corpus_id).first()
        while corpus.status != 'analysing':
            db.session.refresh(corpus)
            socketio.sleep(3)
        analysis_server = '{}_analysis_container{}'.format(corpus.creator.username, corpus.id)
        analysis_client = CQiClient(host=analysis_server)
        analysis_clients[session_id] = analysis_client
        analysis_client.ctrl_connect('opaque', 'opaque')
        while session_id in connected_sessions:
            logger.warning(analysis_client.ctrl_ping())
            socketio.sleep(3)
        analysis_client.ctrl_bye()
        analysis_clients.pop(session_id, None)
        corpus.status = 'stop analysis'
        db.session.commit()
